"""
:作者: 陈思祥

:时间: 2021年4月

:概述:
    当前模块是WordMarker的核心模块，可以读写csv文件，操纵数据库，读写docx文件等。

    1. ``wordmarker.templates.pdbc_template``

        ::

            操纵数据库的模板。

    2. ``wordmarker.templates.csv_template``

        ::

            读写csv文件的模板。

    3. ``wordmarker.templates.word_template``

        ::

            读写docx文件的模板。
"""
from wordmarker.templates.pdbc_template import PdbcTemplate
from wordmarker.templates.pdbc_template import PdbcHelper
from wordmarker.templates.csv_template import CsvTemplate
from wordmarker.templates.csv_template import CsvHelper
from wordmarker.templates.word_template import WordTemplate
from wordmarker.templates.word_template import ImgHelper, DocxHelper



from abc import ABCMeta, abstractmethod


class Context(metaclass=ABCMeta):
    """
    ::

        抽象类：所有上下文的公共父类
    """

    @abstractmethod
    def _init(self):
        """
        ::

            初始化上下文
        """
        pass



"""
:作者: 陈思祥

:时间: 2021年4月

:概述:
    当前模块使用工厂和生成器模式来管理对象的创建。

    1. ``wordmarker.creatives.factory``

        ::

            工厂模块。

    2. ``wordmarker.creatives.builder``

        ::

            生成器模块。
"""
from wordmarker.creatives.factory import BeanFactory
from wordmarker.creatives.factory import FactoryBean
from wordmarker.creatives.builder import AbstractBuilder



import os
from chardet import UniversalDetector
from wordmarker.contexts import SystemContext
from wordmarker.utils import log


class Resource(SystemContext):
    """
    ::

        资源类，包含与资源相关的属性和判断的方法
    """

    @log
    def __init__(self, path, loader):
        self.__path = path
        self.__loader = loader

    def is_file(self):
        """
        .. note::

            判断是否为文件

        :return: - 是文件，返回True

                 - 不是文件，返回False
        """
        return os.path.isfile(self.__path)

    def exists(self):
        """
        .. note::

            判断文件或目录是否存在

        :return: - 存在，返回True

                 - 不存在，返回False
        """
        return os.path.exists(self.__path)

    def get_file_name(self):
        """
        .. note::

            获取文件名

        :return: - 是文件，返回文件的名字

                 - 是目录，返回当前目录下的所有文件的文件名
        """
        if self.exists():
            if self.is_file():
                path, file_name = os.path.split(self.__path)
                return file_name
            else:
                return os.listdir(self.__path)
        else:
            self._logger.error("文件或目录不存在")

    def get_file(self):
        """
        .. note::

            获取文件，返回文件的绝对路径

        :return: - 是文件，返回文件的绝对路径

                 - 是目录，返回当前目录下所有文件的绝对路径
        """
        if self.exists():
            if self.is_file():
                return os.path.abspath(self.__path)
            else:
                file_list = []
                for i in os.listdir(self.__path):
                    file_list.append(os.path.abspath(self.__path + self.path_separator + i))
                return file_list
        else:
            self._logger.error("文件或目录不存在")

    def get_dir(self):
        """
        .. note::

            获取目录，返回目录的绝对路径

        :return: - 是目录，返回目录的绝对路径

                 - 是文件，返回文件所在的目录
        """
        if self.exists():
            if not self.is_file():
                return os.path.abspath(self.__path)
            else:
                path, file_name = os.path.split(self.__path)
                return path
        else:
            self._logger.error("文件或目录不存在")

    def get_file_encoding(self):
        """
        .. note::

            获取文件的编码

        :return: - 是文件，获取文件的编码，返回一个文件编码的字符串

                 - 是目录，获取当前目录下所有文件的编码，返回一个字典

                    - key为文件的绝对路径

                    - value为文件的编码
        """
        if self.exists():
            if self.is_file():
                try:
                    with open(self.get_file(), 'rb') as f:
                        detector = UniversalDetector()
                        for line in f.readlines():
                            detector.feed(line)
                            if detector.done:
                                break
                        detector.close()
                        result: dict = detector.result
                        return result.get('encoding')
                except (IOError, ValueError, TypeError) as exc:
                    self._logger.exception("获取文件的编码失败", exc_info=exc)
            else:
                file_list = self.get_file()
                file_encoding = {}
                try:
                    detector = UniversalDetector()
                    for file in file_list:
                        detector.reset()
                        with open(file, 'rb') as f:
                            for line in f.readlines():
                                detector.feed(line)
                                if detector.done:
                                    break
                        detector.close()
                        result: dict = detector.result
                        file_encoding[file] = result.get('encoding')
                    return file_encoding
                except (IOError, ValueError, TypeError) as exc:
                    self._logger.exception("获取文件的编码失败", exc_info=exc)
        else:
            self._logger.error("文件或目录不存在")

    def get_file_name_prefix_suffix(self):
        """
        .. note::

            获取文件的前缀和后缀

        :return: - 是文件，获取文件的前缀和后缀，返回一个元组

                 - 是目录，获取目录下所有文件的前缀和后缀，返回一个字典

                    - key为文件名

                    - value为由文件的前缀和后缀组成的元组
        """
        if self.exists():
            if self.is_file():
                file_name = self.get_file_name()
                return os.path.splitext(file_name)
            else:
                p_s_dict = {}
                file_name_list = self.get_file_name()
                for file_name in file_name_list:
                    p_s_dict[file_name] = os.path.splitext(file_name)
                return p_s_dict
        else:
            self._logger.error("文件或目录不存在")

    def get_loader(self):
        """
        .. note::

            获取加载当前资源的加载器

        :return: - 加载器
        """
        return self.__loader



from abc import ABCMeta, abstractmethod


class Formatter(metaclass=ABCMeta):
    """
    ::

        格式化的抽象类
    """

    @abstractmethod
    def format(self, *args):
        """
        .. note::

            格式化数据

        :param args: 数据
        :return: - 格式化后的数据
        """
        pass


class SqlFormatter(Formatter):
    """
    ::

        格式化sql语句
    """

    def format(self, sql):
        """
        .. note::

            格式化用户输入的sql语句

            例如：

                .. code-block:: sql
                    :linenos:

                    -- 输入 --
                    select * from t_user where username=? and password=?
                    -- 输出 --
                    select * from t_user where username=:a and password=:b

        .. caution::

            拼接的 ``:a`` ``:b`` 使用的是26个字母，也就是说一次查询的 ``?`` ，不能超过26个。

        :param sql: sql语句
        :return: - 格式化后的sql语句
        """
        sql_list = list(sql)
        index = 0
        n = 97
        for i in sql_list:
            if i == '?':
                sql_list[index] = ':' + chr(n)
                n += 1
            index += 1
        return ''.join(sql_list)



import logging
import os
import shutil
from docxtpl import DocxTemplate, R
from wordmarker.contexts import YamlContext, SystemContext
from wordmarker.creatives import FactoryBean, AbstractBuilder
from wordmarker.loaders import DefaultResourceLoader
from wordmarker.utils import PathUtils, log


class DocxHelper(DefaultResourceLoader):
    """
    ::

        通过读取配置文件，获取docx文件模板的相关信息
    """
    __docx_helper = None

    def __init__(self):
        super().__init__()
        self.__yaml_context: YamlContext = FactoryBean().get_bean("yaml_context")
        self._docx_in_path = self._get_docx_in_path()
        self._docx_out_path = self._get_docx_out_path()
        self._docx = self._get_docx()

    def _get_docx_in_path(self):
        """
        .. note::

            通过读取yaml文件的 ``data.docx.input.path`` 属性，获取输入的docx文件或目录的绝对路径

        :return: - docx文件或目录的绝对路径
        """
        prop = "data.docx.input.path"
        value = self.__yaml_context.get_value(prop)
        docx_in_path = PathUtils(self.__yaml_context.path, value).get_relative_path()
        self._docx_in_path = docx_in_path
        return docx_in_path

    def _get_docx_out_path(self):
        """
        .. note::

            通过读取yaml文件的 ``data.docx.output.dir`` 属性，获取输出的docx目录的绝对路径

        :return: - docx目录的绝对路径
        """
        prop = "data.docx.output.dir"
        value = self.__yaml_context.get_value(prop)
        return PathUtils(self.__yaml_context.path, value).get_relative_path()

    def _get_docx(self):
        """
        .. note::

            获取docx文件的绝对路径

        :return: - yaml文件中 ``data.docx.input.path`` 是目录，返回当前目录下docx文件的绝对路径

                 - yaml文件中 ``data.docx.input.path`` 是文件，返回docx文件的绝对路径
        """
        docx = self.get_resource(self._docx_in_path).get_file()
        if not self.get_resource(self._docx_in_path).is_file():
            # 是目录
            docx = PathUtils.filter_file(docx, ['.docx'])
        else:
            # 是文件
            docx = PathUtils.filter_file(docx, ['.docx'])[0]
        return docx

    def get_docx_file_name(self):
        """
        .. note::

            获取docx文件的文件名

        :return: - 是docx文件，返回文件的名字

                 - 是目录，返回当前目录下的所有docx文件的文件名
        """
        docx = self._docx
        if type(docx) is list:
            temp_list = []
            for item in docx:
                temp_list.append(os.path.split(item)[1])
            return temp_list
        else:
            return os.path.split(docx)[1]

    @property
    def docx_in_path(self):
        """
        .. note::

            通过读取yaml文件的 ``data.docx.input.path`` 属性，获取输入的docx文件或目录的绝对路径

        :return: - docx文件或目录的绝对路径
        """
        return self._docx_in_path

    @property
    def docx_out_path(self):
        """
        .. note::

            通过读取yaml文件的 ``data.docx.output.dir`` 属性，获取输出的docx目录的绝对路径

        :return: - docx目录的绝对路径
        """
        return self._docx_out_path

    @property
    def docx(self):
        """
        .. note::

            获取docx文件的绝对路径

        :return: - yaml文件中 ``data.docx.input.path`` 是目录，返回当前目录下docx文件的绝对路径

                 - yaml文件中 ``data.docx.input.path`` 是文件，返回docx文件的绝对路径
        """
        return self._docx

    def __new__(cls, *args, **kwargs):
        if cls.__docx_helper is None:
            cls.__docx_helper = object.__new__(cls)
        return cls.__docx_helper


class ImgHelper(SystemContext):
    """
    ::

        通过读取配置文件，获取img文件的相关信息
    """
    __img_helper = None

    def __init__(self):
        super().__init__()
        self.__yaml_context: YamlContext = FactoryBean().get_bean("yaml_context")
        self.__img_out_path = self._get_img_out_path()

    def _get_img_out_path(self):
        prop = "data.img.output.dir"
        value = self.__yaml_context.get_value(prop)
        return PathUtils(self.__yaml_context.path, value).get_relative_path()

    def get_img_file(self, img_name):
        """
        .. note::

            根据图片的名字，获取图片的绝对路径

        .. warning::

            必须先将图片输出到输出目录下，才能获取到

        :param img_name: 图片的名字
        :return: - 图片的绝对路径
        """
        final_path = self.__img_out_path + self.path_separator + img_name
        return final_path

    @property
    def img_out_path(self):
        """
        .. note::

            通过读取yaml文件的 ``data.img.output.dir`` 属性，获取输出的img目录的绝对路径

        :return: - img目录的绝对路径
        """
        return self.__img_out_path

    def clear_img(self):
        """
        .. note::

            清除 ``data.img.output.dir`` 属性对应的img的输出目录下的所有文件和目录

        """
        if os.path.exists(self.__img_out_path):
            shutil.rmtree(self.__img_out_path)
            os.mkdir(self.__img_out_path)

    def __new__(cls, *args, **kwargs):
        if cls.__img_helper is None:
            cls.__img_helper = object.__new__(cls)
        return cls.__img_helper


class TextHelper:
    """
    ::

        通过读取配置文件，获取文本yaml文件的相关信息
    """
    __text_helper = None

    def __init__(self):
        super().__init__()
        self.__yaml_context: YamlContext = FactoryBean().get_bean("yaml_context")
        self.__text_in_path = self._get_text_in_path()
        self.__text_context = YamlContext(self.text_in_path)
        self.__yaml_singleton = {}

    def _get_text_in_path(self):
        prop = "data.text.input.path"
        value = self.__yaml_context.get_value(prop)
        return PathUtils(self.__yaml_context.path, value).get_relative_path()

    @property
    def text_in_path(self):
        """
        .. note::

            通过读取yaml文件的 ``data.text.input.path`` 属性，获取输入的文本yaml文件的绝对路径

        :return: - 文本yaml文件的绝对路径
        """
        return self.__text_in_path

    def get_value(self, prop):
        """
        .. note::

            从yaml字典中，根据属性获取对应的值

            加载多个yaml文件，排在后面的文件里的值，会覆盖前面的文件里的值

        :param prop: 属性，用 ``.`` 分隔，例如，``pdbc.engine.url``
        :return: - yaml字典中对应的值
        """
        return self.__text_context.get_value(prop)

    def get_yaml(self) -> dict:
        """
        .. note::

            获取从yaml文件中读取的数据，类型为dict

        :return: - path为文件，返回一个字典，内容为yaml文件的内容

                 - path为目录，返回一个嵌套的字典

                    - key为yaml文件的绝对路径

                    - value为yaml文件的内容，是一个字典
        """
        return self.__text_context.get_yaml()

    def get_yaml_singleton(self):
        """
        .. note::

            获取从 ``data.text.input.path`` 属性中对应的路径下所有yaml文件中的数据，类型为dict

            加载多个yaml文件，排在后面的文件里的值，会覆盖前面的文件里的值

        :return: - 返回一个字典，内容为所有yaml文件的内容
        """
        for i in self.get_yaml().values():
            self.__yaml_singleton.update(i)
        return self.__yaml_singleton

    def get_yaml_singleton_str(self):
        """
        .. note::

            获取从 ``data.text.input.path`` 属性中对应的路径下所有yaml文件中的数据，类型为str，内容为一个字典

            加载多个yaml文件，排在后面的文件里的值，会覆盖前面的文件里的值

        :return: - 返回一个字符串，内容为一个字典，内容为所有yaml文件的内容
        """
        return str(self.get_yaml_singleton())

    def __new__(cls, *args, **kwargs):
        if cls.__text_helper is None:
            cls.__text_helper = object.__new__(cls)
        return cls.__text_helper


class WordTemplate(AbstractBuilder, TextHelper, ImgHelper, DocxHelper):
    """
    ::

        操作docx文件的模板
    """

    @log
    def __init__(self, tpl_name=None):
        super().__init__()
        self.content = {'page_break': R('\f')}
        self.__tpl_name = tpl_name
        self.__tpl = DocxTemplate(self.__get_tpl_name_abs())

    def __get_tpl_name_abs(self):
        if type(self._get_docx()) is list:
            if self.__tpl_name is None:
                self._logger: logging.Logger
                self._logger.error("请输入模板的文件名")
            else:
                for i in self.get_docx_file_name():
                    if self.__tpl_name == i:
                        for j in self._get_docx():
                            if self.__tpl_name == os.path.split(j)[1]:
                                return j
                    else:
                        self._logger.error("模板的文件名不正确")
                        break
        else:
            return self._docx

    def append(self, content):
        """
        .. note::

            添加其他的content到全局的content中

        :param content: 其他的content，类型是 ``dict``
        :return: - self
        """
        self.content.update(content)
        return self

    def build(self, file_name=None):
        """
        .. note::

            创建docx文件

        :param file_name: docx文件的文件名
        """
        if file_name:
            self.__tpl.render(self.content)
            path = self.docx_out_path + self.path_separator + os.path.splitext(file_name)[0]
            if os.path.exists(path):
                shutil.rmtree(path)
            os.mkdir(path)
            img_path = path + self.path_separator + 'img'
            if os.path.exists(img_path):
                shutil.rmtree(img_path)
            shutil.copytree(self.img_out_path, img_path)
            self.__tpl.save(path + self.path_separator + file_name)
        else:
            self._logger: logging.Logger
            self._logger.error('请输入输出的docx文件的文件名')

    @property
    def tpl(self):
        """
        .. note::

            获取 ``DocxTemplate`` 对象，

        .. tip::

            ``DocxTemplate`` 对象的详细信息，请访问 `python-docx-template的文档 <https://docxtpl.readthedocs.io/>`_

        :return: - ``DocxTemplate`` 对象
        """
        return self.__tpl



from abc import ABCMeta, abstractmethod
from collections import KeysView


class AbstractBeanFactory(metaclass=ABCMeta):
    """
    ::

        BeanFactory的抽象类
    """

    @abstractmethod
    def get_bean_names(self):
        """
        .. note::

            获取工厂中所有bean实例对应的名字

        :return: - 所有的bean实例对应的名字
        """
        pass

    @abstractmethod
    def get_bean(self, name: str):
        """
        .. note::

            根据bean的名字获取bean实例

        :param name: 名字
        :return: - bean实例
        """
        pass

    @abstractmethod
    def contain_bean(self, name: str):
        """
        .. note::

            判断工厂中是否包含某个bean实例

        :param name: 名字
        :return: - 包含，返回True
                 - 不包含，返回False
        """
        pass

    @abstractmethod
    def get_type(self, name: str):
        """
        .. note::

            获取bean实例的类型

        :param name: 名字
        :return: - 工厂中存在bean实例，返回bean实例的类型
                 - 不存在，返回 None 对应的类型 NoneType
        """
        pass


class BeanFactory(AbstractBeanFactory):
    """
    ::

        AbstractBeanFactory的实现类
    """
    __factory_bean = None
    _beans = {}

    def get_bean_names(self) -> KeysView:
        """
        .. note::

            获取工厂中所有bean实例对应的名字

        :return: - 所有的bean实例对应的名字
        """
        return self._beans.keys()

    def get_bean(self, name: str):
        """
        .. note::

            根据bean的名字获取bean实例

        :param name: 名字
        :return: - bean实例
        """
        return self._beans.get(name)

    def contain_bean(self, name: str):
        """
        .. note::

            判断工厂中是否包含某个bean实例

        :param name: 名字
        :return: - 包含，返回True
                 - 不包含，返回False
        """
        return name in self._beans.keys()

    def get_type(self, name: str):
        """
        .. note::

            获取bean实例的类型

        :param name: 名字
        :return: - 工厂中存在bean实例，返回bean实例的类型
                 - 不存在，返回 None 对应的类型 NoneType
        """
        if self.contain_bean(name):
            return type(self._beans.get(name))
        else:
            return type(None)

    def __new__(cls, *args, **kwargs):
        if cls.__factory_bean is None:
            cls.__factory_bean = object.__new__(cls)
        return cls.__factory_bean


class FactoryBean(BeanFactory):
    """
    ::

        BeanFactory的子类

        通过add_bean方法，将bean实例添加到工厂
    """
    __factory_bean = None

    def __init__(self):
        if not self.contain_bean("factory_bean"):
            # 将自身 factory_bean 添加到工厂
            self.add_bean("factory_bean", self)

    def __new__(cls, *args, **kwargs):
        if cls.__factory_bean is None:
            cls.__factory_bean = object.__new__(cls)
        return cls.__factory_bean

    def add_bean(self, name: str, bean):
        """
        .. note::

            将bean实例添加到工厂

        :param name: 名字
        :param bean: bean实例
        """
        self._beans[name] = bean



import pandas as pd
import sqlalchemy as sa
from abc import ABCMeta, abstractmethod
from copy import deepcopy
from logging import Logger
from typing import Iterator, Union, Optional
from pandas import DataFrame
from pandas.core.generic import bool_t
from sqlalchemy import create_engine
from sqlalchemy.engine import Engine
from sqlalchemy.exc import ArgumentError
from wordmarker.contexts import YamlContext
from wordmarker.creatives import FactoryBean
from wordmarker.data.formatter import SqlFormatter
from wordmarker.utils import log


class PdbcOperations(metaclass=ABCMeta):
    """
    ::

        数据库的相关操作的抽象类
    """

    @abstractmethod
    def execute(self, sql):
        """
        .. note::

            执行sql语句，建议sql类型为DDL（数据定义语言）时候使用

        :param sql: sql语句
        """
        pass

    @abstractmethod
    def update(self, sql):
        """
        .. note::

            更新数据库

        :param sql: sql语句
        """
        pass

    @abstractmethod
    def query(self, sql):
        """
        .. note::

            查询数据库

        :param sql: sql语句
        :return: - 查询结果
        """
        pass


class PdbcHelper:
    """
    ::

        通过读取配置文件获取数据库的信息，进而建立连接
    """
    __engine_dict = {}  # sqlalchemy.create_engine方法中传入的值的字典
    __pdbc_helper = None

    @log
    def __init__(self):
        self.__yaml_context: YamlContext = FactoryBean().get_bean("yaml_context")
        self._get_url()
        self._get_echo()
        self._get_encoding()
        self._get_pool_size()
        self._get_pool_timeout()
        self._get_pool_recycle()
        self._get_max_overflow()
        self._get_echo_pool()
        self._engine = None
        self.set_engine()

    def _get_url(self):
        """
        获取配置文件中url的值
        """
        prop = "pdbc.engine.url"
        value = self.__yaml_context.get_value(prop)
        prop = prop.split('.')[2]
        self.__engine_dict[prop] = value
        return value

    def _get_echo(self):
        """
        获取配置文件中echo的值
        """
        prop = "pdbc.engine.echo"
        value = self.__yaml_context.get_value(prop)
        prop = prop.split('.')[2]
        self.__engine_dict[prop] = value
        return value

    def _get_encoding(self):
        """
        获取配置文件中encoding的值
        """
        prop = "pdbc.engine.encoding"
        value = self.__yaml_context.get_value(prop)
        prop = prop.split('.')[2]
        self.__engine_dict[prop] = value
        return value

    def _get_pool_size(self):
        """
        获取配置文件中pool_size的值
        """
        prop = "pdbc.engine.pool_size"
        value = self.__yaml_context.get_value(prop)
        prop = prop.split('.')[2]
        self.__engine_dict[prop] = value
        return value

    def _get_max_overflow(self):
        """
        获取配置文件中max_overflow的值
        """
        prop = "pdbc.engine.max_overflow"
        value = self.__yaml_context.get_value(prop)
        prop = prop.split('.')[2]
        self.__engine_dict[prop] = value
        return value

    def _get_pool_recycle(self):
        """
        获取配置文件中pool_recycle的值
        """
        prop = "pdbc.engine.pool_recycle"
        value = self.__yaml_context.get_value(prop)
        prop = prop.split('.')[2]
        self.__engine_dict[prop] = value
        return value

    def _get_pool_timeout(self):
        """
        获取配置文件中pool_timeout的值
        """
        prop = "pdbc.engine.pool_timeout"
        value = self.__yaml_context.get_value(prop)
        prop = prop.split('.')[2]
        self.__engine_dict[prop] = value
        return value

    def _get_echo_pool(self):
        """
        获取配置文件中echo_pool的值
        """
        prop = "pdbc.engine.echo_pool"
        value = self.__yaml_context.get_value(prop)
        prop = prop.split('.')[2]
        self.__engine_dict[prop] = value
        return value

    def set_engine(self, **kwargs):
        """
        .. note::

            设置引擎

        .. tip::

            你可以在调用PdbcTemplate内的方法之前，设置引擎需要的其他参数（不包括配置文件内的参数）

        :param kwargs: 除去配置文件中其他的值，采用key=value的形式
        :return: - engine对象
        """
        self._logger: Logger
        url = self._get_url()

        if url is None:
            self._logger.error("yaml文件中的url为空")
        else:
            temp_dict = deepcopy(self.__engine_dict)
            temp_dict.pop("url")
            try:
                temp_list = []
                for key, value in temp_dict.items():
                    if value is None:
                        temp_list.append(key)
                for key in temp_list:
                    temp_dict.pop(key)
                if len(temp_dict) > 0:
                    self.__engine_dict.update(kwargs)
                    engine = create_engine(url, **temp_dict, **kwargs)
                else:
                    engine = create_engine(url)
                self._engine = engine
            except (SyntaxError, TypeError) as exc:
                self._logger.exception("通过函数传入的键值对，与yaml配置中的键值对，冲突", exc_info=exc)
            except ArgumentError as exc:
                self._logger.exception("无法解析yaml配置中的url", exc_info=exc)

    @property
    def engine(self) -> Engine:
        """
        .. note::
            获取引擎对象

        :return: - 引擎对象engine
        """
        return self._engine

    @property
    def engine_dict(self):
        """
        .. note::

            获取引擎对象中设置的值

        :return: - 引擎对象中设置的值

                    - key为属性

                    - value为对应的值
        """
        return self.__engine_dict

    def __new__(cls, *args, **kwargs):
        if cls.__pdbc_helper is None:
            cls.__pdbc_helper = object.__new__(cls)
        return cls.__pdbc_helper


class PdbcTemplate(PdbcOperations, PdbcHelper, SqlFormatter):
    """
    ::

        操作数据库的模板
    """

    @log
    def __init__(self):
        super().__init__()

    def execute(self, sql, *args, **kwargs):
        """
        .. note::

            使用sqlalchemy中的方法执行sql，建议sql类型为DDL（数据定义语言）时候使用

        .. tip::

            如果sql为 ``select``，建议使用query方法

            如果sql为 ``update`` ，``delete`` ，``insert`` ，建议使用update方法

        :param sql: sql语句
        """
        with self.engine.connect() as con:
            return con.execute(sql, *args, **kwargs)

    def update(self, sql, *args):
        """
        .. note::

            更新数据库

        :param sql: sql语句
        :param args: 问号对应的值
        """
        sql = self.format(sql)
        key_list = []
        value_list = [*args]
        params = {}

        for i in range(97, 97 + len(args)):
            key_list.append(chr(i))

        index = 0
        for key in key_list:
            params[key] = value_list[index]
            index += 1

        with self.engine.connect() as con:
            con.execute(sa.text(sql), params)
            self._logger: Logger
            self._logger.info("更新数据成功")

    def query(self, sql, *args) -> Union[DataFrame, Iterator[DataFrame]]:
        """
        .. note::

            查询数据库

        :param sql: sql语句
        :param args: 问号对应的值
        :return: - 查询的数据
        """
        sql = self.format(sql)
        key_list = []
        value_list = [*args]
        params = {}

        for i in range(97, 97 + len(args)):
            key_list.append(chr(i))

        index = 0
        for key in key_list:
            params[key] = value_list[index]
            index += 1

        with self.engine.connect() as con:
            data = pd.read_sql(sa.text(sql), con, params=params)
            return data

    def query_table(self,
                    table_name,
                    schema=None,
                    index_col=None,
                    coerce_float=True,
                    parse_dates=None,
                    columns=None,
                    chunksize: Optional[int] = None) -> Union[DataFrame, Iterator[DataFrame]]:
        """
        .. note::

            使用 ``pandas.read_sql_table`` 方法，读取整张表的数据

        .. tip::

            `query_table的参数详情 <https://pandas.pydata.org/docs/reference/api/pandas.read_sql_table.html#pandas.read_sql_table>`_ ，
            请访问 `pandas官网`_ ，了解更多信息

        :return: - 查询的数据
        """
        with self.engine.connect() as con:
            return pd.read_sql_table(table_name,
                                     con,
                                     schema=schema,
                                     index_col=index_col,
                                     coerce_float=coerce_float,
                                     parse_dates=parse_dates,
                                     columns=columns,
                                     chunksize=chunksize
                                     )

    def update_table(self,
                     data: DataFrame,
                     name: str,
                     schema=None,
                     if_exists: str = "replace",
                     index: bool_t = False,
                     index_label=None,
                     chunksize=None,
                     dtype=None,
                     method=None):
        """
        .. note::

            使用 ``pandas.to_sql`` 方法，将数据写入数据库中的一张表中

        .. tip::

            `update_table的参数详情 <https://pandas.pydata.org/docs/reference/api/pandas.DataFrame.to_sql.html#pandas.DataFrame.to_sql>`_ ，
            请访问 `pandas官网`_ ，了解更多信息

        :param data: 数据框
        """
        with self.engine.connect() as con:
            data.to_sql(name,
                        con=con,
                        schema=schema,
                        if_exists=if_exists,
                        index=index,
                        index_label=index_label,
                        chunksize=chunksize,
                        dtype=dtype,
                        method=method)



import os.path
from wordmarker.contexts import Context, YamlContext
from wordmarker.creatives import BeanFactory, FactoryBean


class WordMarkerContext(Context):
    """
    ::

        应用上下文，你可以从上下文中获取到：

        bean_factory：工厂里存放的bean实例的相关信息

        yaml_context：加载的yaml文件的相关信息

    """
    __word_marker_context = None
    __bean_factory = None  # getter方法获取
    __factory_bean = None  # 工厂获取
    __yaml_context = None  # getter方法获取
    __xml_context = None  # getter方法获取

    def __init__(self, resource):
        """
        :param resource: 路径必须为根路径或绝对路径
        """
        self.__resource = os.path.abspath(resource)
        if (not self.__factory_bean) and (not self.__bean_factory):  # 避免工厂多次初始化
            self._init()
        else:
            self.__init_other_context()

    def _init(self):
        self.__init_factory()
        self.__init_other_context()

    def __init_factory(self):
        """
        初始化工厂
        """
        self.__init_factory_bean()
        self.__init_bean_factory()

    def __init_other_context(self):
        """
        初始化其他上下文
        """
        self.__init_yaml_context()

    def __init_factory_bean(self):
        """
        初始化FactoryBean，用来添加bean实例
        """
        self.__factory_bean = FactoryBean()

    def __init_bean_factory(self):
        """
        初始化BeanFactory，用来获取bean的信息
        """
        self.__bean_factory = BeanFactory()

    def __init_yaml_context(self):
        """
        初始化yaml_context
        """
        # 1. 获取资源，然后加载资源
        self.__yaml_context = YamlContext(self.__resource)
        # 2. 将yaml_context添加到工厂
        self.__factory_bean.add_bean("yaml_context", self.__yaml_context)

    @property
    def bean_factory(self):
        """
        .. note::

            获取bean工厂

        :return: - bean工厂
        """
        return self.__bean_factory

    @property
    def yaml_context(self):
        """
        .. note::

            获取yaml文件的上下文

        :return: - yaml文件的上下文
        """
        return self.__yaml_context

    def __new__(cls, *args, **kwargs):
        if cls.__word_marker_context is None:
            cls.__word_marker_context = object.__new__(cls)
        return cls.__word_marker_context



"""
:作者: 陈思祥

:时间: 2021年4月

:概述:
    当前模块包含整个应用的上下文。

    1. ``wordmarker.contexts.context``

        ::

            上下文的抽象类，所有的上下文都要继承它。

    2. ``wordmarker.contexts.system_context``

        ::

            系统上下文，获取和系统有关的属性。例如，路径分隔符，文件分隔符等。

    3. ``wordmarker.contexts.yaml_context``

        ::

            yaml文件的上下文，解析yaml文件。根据key值返回yaml文件中对应的value值。

    4. ``wordmarker.contexts.application_context``

        ::

            应用上下文，用来初始化工厂和其他的上下文。

            一般初始化，应用上下文WordMarkerContext来初始化整个应用。
"""
from wordmarker.contexts.context import Context
from wordmarker.contexts.system_context import SystemContext
from wordmarker.contexts.yaml_context import YamlContext
from wordmarker.contexts.application_context import WordMarkerContext



import logging
from wordmarker.utils import log


class YamlUtils:
    """
    ::

        yaml文件工具类
    """

    @log
    def __init__(self):
        pass

    def get_value(self, yaml_dict, prop_list, temp_prop, file_name):
        """
        .. note::

            递归调用获取yaml文件中属性对应的值

        :param yaml_dict: yaml文件中的数据，以字典形式存放
        :param prop_list: 属性列表，例如： ``pdbc.engine.url`` -> ``[pdbc, engine, url]``
        :param temp_prop: 临时属性，在日志中进行提示，例如： ``pdbc.engine.url``
        :param file_name: yaml文件名
        :return: - yaml文件中属性对应的值
        """
        prop_list = prop_list
        if (len(prop_list) == 1) and (len(prop_list[0]) == 0):
            self._logger.error("输入的属性的值为空，在{file_name}中获取不到".format(file_name=file_name))
        else:
            value = yaml_dict.get(prop_list.pop(0))
            if type(value) == dict:
                return self.get_value(value, prop_list, temp_prop, file_name)
            else:
                if value is not None:
                    self._logger: logging.Logger
                    self._logger.info(
                        "{file_name}文件中，属性 {prop} 对应的值获取成功".format(file_name=file_name, prop=temp_prop))
                    return value
                else:
                    self._logger: logging.Logger
                    self._logger.warning(
                        "{file_name}文件中，属性 {prop} 对应的值为空。".format(file_name=file_name, prop=temp_prop))



import os
import shutil
import click
from ruamel.yaml import YAML

path_separator = os.path.sep

inp_standard = """
# ======================================================================================================

# ----------------------配置数据库信息
#
# 详细信息请查看 https://docs.sqlalchemy.org/en/14/core/engines.html
# url:
#    postgresql: "postgresql://username:password@localhost:5432/database"
#    mysql: mysql+pymysql://username:password@localhost:3306/database
# echo: 是否开启日志
# encoding: 编码，默认utf-8
# pool_size: 连接池大小
# max_overflow: 连接的数量，以允许在连接池"溢出"
# pool_recycle: 在给定的秒数过去之后，此设置将导致池回收连接。默认为-1。例如，设置为3600表示一小时后将回收连接
# pool_timeout: 放弃从池中获得连接之前要等待的秒数
# echo_pool: 是否开启日志
pdbc:
  engine:
    url: {url}
    echo: false
    encoding: utf-8
    pool_size: 5
    max_overflow: 10
    pool_recycle: -1
    pool_timeout: 30.0
    echo_pool: false

# ======================================================================================================

# ----------------------配置csv文件信息
#
# path: 文件的输入路径，可以是文件，也可以是目录
#       路径分隔符为 '/' 或 '\\'
#       path的值为相对于当前yaml文件的路径
# dir: 文件的输出目录
#      路径分隔符为 '/' 或 '\\'
#      dir的值为相对于当前yaml文件的路径
data:
  csv:
    input:
      path: data/in
    output:
      dir: data/out
  docx:
    input:
      path: template/in
    output:
      dir: template/out
  img:
    output:
      dir: img
  text:
    input:
      path: text

# ======================================================================================================
"""

inp_custom = """
# ======================================================================================================

# ----------------------配置数据库信息
#
# 详细信息请查看 https://docs.sqlalchemy.org/en/14/core/engines.html
# url:
#    postgresql: "postgresql://username:password@localhost:5432/database"
#    mysql: mysql+pymysql://username:password@localhost:3306/database
# echo: 是否开启日志
# encoding: 编码，默认utf-8
# pool_size: 连接池大小
# max_overflow: 连接的数量，以允许在连接池"溢出"
# pool_recycle: 在给定的秒数过去之后，此设置将导致池回收连接。默认为-1。例如，设置为3600表示一小时后将回收连接
# pool_timeout: 放弃从池中获得连接之前要等待的秒数
# echo_pool: 是否开启日志
pdbc:
  engine:
    url: {url}
    echo: false
    encoding: utf-8
    pool_size: 5
    max_overflow: 10
    pool_recycle: -1
    pool_timeout: 30.0
    echo_pool: false

# ======================================================================================================

# ----------------------配置csv文件信息
#
# path: 文件的输入路径，可以是文件，也可以是目录
#       路径分隔符为 '/' 或 '\\'
#       path的值为相对于当前yaml文件的路径
# dir: 文件的输出目录
#      路径分隔符为 '/' 或 '\\'
#      dir的值为相对于当前yaml文件的路径
data:
  csv:
    input:
      path: ''
    output:
      dir: ''
  docx:
    input:
      path: ''
    output:
      dir: ''
  img:
    output:
      dir: ''
  text:
    input:
      path: ''

# ======================================================================================================
"""


def generate_yaml(url, flag):
    yaml = YAML()
    if flag:
        content = yaml.load(inp_standard.format(url=url))
    else:
        content = yaml.load(inp_custom.format(url=url))
    current_dir = os.path.abspath(os.curdir)
    with open(current_dir + path_separator + "config.yaml", encoding='utf-8', mode='w') as f:
        yaml.dump(content, f)


@click.command()
def main():
    click.echo()
    click.echo("欢迎使用 WordMarker 的脚手架，请按照提示完成项目的构建。")
    url = database()
    flag = input_output()
    generate_yaml(url, flag)
    click.echo()
    end()


def database():
    dialect = database_dialect()
    return database_config(dialect)


def database_dialect():
    click.echo("-------------------------------------------------")
    click.echo("配置数据库")
    click.echo("-------------------------------------------------")
    click.echo("1. PostgreSQL")
    click.echo("2. MySql")
    click.echo("3. Oracle")
    click.echo("4. Microsoft")
    return click.prompt("数据库的类型，默认为", type=click.IntRange(1, 4), default=1)


def database_config(dialect):
    if dialect == 1:
        username = database_username("postgres")
        password = database_password()
        host = database_host("localhost")
        port = database_port(5432)
        name = database_name()
        return database_url(dialect, username, password, host, port, name)
    elif dialect == 2:
        username = database_username("root")
        password = database_password()
        host = database_host("localhost")
        port = database_port(3306)
        name = database_name()
        return database_url(dialect, username, password, host, port, name)
    elif dialect == 3:
        username = database_username("")
        password = database_password()
        host = database_host("localhost")
        port = database_port(1521)
        name = database_name()
        return database_url(dialect, username, password, host, port, name)
    elif dialect == 4:
        username = database_username("")
        password = database_password()
        host = database_host("localhost")
        port = database_port(1433)
        name = database_name()
        return database_url(dialect, username, password, host, port, name)


def database_username(username):
    return click.prompt("用户名，默认为", type=str, default=username)


def database_password():
    return click.prompt("密码，默认为", type=str, default="123456")


def database_host(host):
    return click.prompt("主机，默认为", type=str, default=host)


def database_port(port):
    return click.prompt("端口，默认为", type=int, default=port)


def database_name():
    return click.prompt("要连接的数据库", type=str)


def database_url(dialect, username, password, host, port, name):
    if dialect == 1:
        return f"postgresql://{username}:{password}@{host}:{port}/{name}"
    elif dialect == 2:
        return f"mysql+pymysql://{username}:{password}@{host}:{port}/{name}"
    elif dialect == 3:
        return f"oracle://{username}:{password}@{host}:{port}/{name}"
    elif dialect == 4:
        return f"mssql+pymssql://{username}:{password}@{host}:{port}/{name}"


def input_output():
    click.echo("-------------------------------------------------")
    click.echo("配置输入输出目录")
    click.echo("-------------------------------------------------")
    if click.confirm("是否启用默认的目录配置，默认为 [Y]", default=True):
        current_dir = os.path.abspath(os.curdir)
        if os.path.exists(current_dir + path_separator + "data"):
            shutil.rmtree(current_dir + path_separator + "data")
        if os.path.exists(current_dir + path_separator + "template"):
            shutil.rmtree(current_dir + path_separator + "template")
        if os.path.exists(current_dir + path_separator + "img"):
            shutil.rmtree(current_dir + path_separator + "img")
        if os.path.exists(current_dir + path_separator + "text"):
            shutil.rmtree(current_dir + path_separator + "text")
        os.makedirs(current_dir + path_separator + "data" + path_separator + "in")
        click.echo("成功创建data/in目录")
        os.makedirs(current_dir + path_separator + "data" + path_separator + "out")
        click.echo("成功创建data/out目录")
        os.makedirs(current_dir + path_separator + "template" + path_separator + "in")
        click.echo("成功创建template/in目录")
        os.makedirs(current_dir + path_separator + "template" + path_separator + "out")
        click.echo("成功创建template/out目录")
        os.makedirs(current_dir + path_separator + "img")
        click.echo("成功创建img目录")
        os.makedirs(current_dir + path_separator + "text")
        click.echo("成功创建text目录")
        return True
    else:
        current_dir = os.path.abspath(os.curdir)
        if os.path.exists(current_dir + path_separator + "data"):
            shutil.rmtree(current_dir + path_separator + "data")
        if os.path.exists(current_dir + path_separator + "template"):
            shutil.rmtree(current_dir + path_separator + "template")
        if os.path.exists(current_dir + path_separator + "img"):
            shutil.rmtree(current_dir + path_separator + "img")
        if os.path.exists(current_dir + path_separator + "text"):
            shutil.rmtree(current_dir + path_separator + "text")
        return False


def end():
    click.echo("成功构建项目 ^_^ !")


if __name__ == '__main__':
    main()



import copy
from wordmarker.contexts.context import Context
from wordmarker.loaders.yaml_resource_loader import YamlResourceLoader
from wordmarker.utils import YamlUtils


class YamlContext(Context, YamlResourceLoader):
    """
    ::

        yaml文件的上下文
    """

    def __init__(self, path):
        super().__init__()
        self.__path = path
        self.__data = None
        self._init()

    def _init(self):
        """
        ::

            初始化yaml上下文

        :return: 从yaml文件中加载的数据，类型为dict
        """
        resource = self.get_resource(self.path)
        self.__data = self.load(resource)

    def get_yaml(self):
        """
        .. note::

            获取从yaml文件中读取的数据，类型为dict

        :return: - path为文件，返回一个字典，内容为yaml文件的内容

                 - path为目录，返回一个嵌套的字典

                    - key为yaml文件的绝对路径

                    - value为yaml文件的内容，是一个字典
        """
        return self.__data

    def get_value(self, prop):
        """
        .. note::

            从yaml字典中，根据属性获取对应的值

            加载多个yaml文件，排在后面的文件里的值，会覆盖前面的文件里的值

        :param prop: 属性，用 ``.`` 分隔，例如，``pdbc.engine.url``
        :return: - yaml字典中对应的值
        """
        prop_list = prop.split('.')
        if self.get_resource(self.path).is_file():
            yaml_dict = self.get_yaml()
            value = YamlUtils().get_value(yaml_dict, prop_list, prop, self.get_resource(self.path).get_file())
            return value
        else:
            yaml = self.get_yaml()
            value = None
            for key, val in yaml.items():
                temp_list = copy.deepcopy(prop_list)
                v = YamlUtils().get_value(val, temp_list, prop, key)
                if v is not None:
                    value = v
            return value

    @property
    def path(self):
        """
        .. note::

            存放yaml文件的路径，可以是文件，也可以是目录

        :return: - yaml文件的路径
        """
        return self.__path

    @path.setter
    def path(self, path):
        self.__path = path



import os
from typing import Union
from wordmarker.contexts import SystemContext


class PathUtils(SystemContext):
    """
    ::

        路径工具类
    """

    def __init__(self, src, tgt):
        self.__src = src
        self.__tgt = tgt

    def get_relative_path(self):
        """
        .. note::

            将两个路径拼接起来

            例子：

                ::

                    src = c:/a/b/c

                    tgt = ../../d/c.txt

                    final = c:/a/d/c.txt

                    -------------------------------

                    src = c:/a/b/c/foo.txt

                    tgt = ../../d/c.txt

                    final = c:/a/d/c.txt

        :return: - 最终路径
        """
        src: str = self.__src
        tgt: str = self.__tgt
        src = os.path.normpath(src)
        tgt = os.path.normpath(tgt)
        if os.path.isfile(src):
            src_dir = os.path.dirname(src)
        else:
            src_dir = src
        src_list = src_dir.split(self.path_separator)
        tgt_list = tgt.split(self.path_separator)
        index = 0
        for i in tgt_list:
            if i == r'..':
                src_list.pop()
                index += 1
        for i in range(0, index):
            tgt_list.pop(0)
        final_list = src_list + tgt_list
        final_path = self.path_separator.join(final_list)
        return final_path

    @staticmethod
    def filter_file(file_list: Union[list, str], suffix_list: Union[list, str]):
        """
        .. note::

            通过后缀名，过滤文件

        :param file_list: 文件或文件列表
        :param suffix_list: 后缀或后缀列表
        :return: - 过滤后的文件或文件列表
        """
        final_list = []
        if type(file_list) is not list:
            file_list = [file_list]
        for item in file_list:
            file_suffix = os.path.splitext(item)[1]
            if type(suffix_list) is not list:
                suffix_list = [suffix_list]
            if file_suffix in suffix_list:
                final_list.append(item)
        return final_list



"""
:作者: 陈思祥

:时间: 2021年4月

:概述:
    当前模块用来加载资源。

    1. ``wordmarker.loaders.default_resource_loader``

        ::

            默认的资源加载器。

    2. ``wordmarker.loaders.yaml_resource_loader``

        ::

            yaml文件的资源加载器。

    3. ``wordmarker.loaders.resource_loader``

        ::

            资源加载器的抽象类。
"""
from wordmarker.loaders.default_resource_loader import DefaultResourceLoader
from wordmarker.loaders.yaml_resource_loader import YamlResourceLoader



"""
:作者: 陈思祥

:时间: 2021年4月

:概述:
    当前模块是工具模块，有操作日志、文件等的工具类

    1. ``wordmarker.utils.logger``

        ::

            日志工具类。

    2. ``wordmarker.utils.yaml``

        ::

            yaml文件工具类。

    3. ``wordmarker.utils.file``

        ::

            文件、路径工具类。
"""
from wordmarker.utils.logger import log
from wordmarker.utils.logger import LoggerFactory
from wordmarker.utils.yaml import YamlUtils
from wordmarker.utils.file import PathUtils



"""
:作者: 陈思祥

:时间: 2021年4月

:概述:
    当前模块用来处理数据和资源。

    1. ``wordmarker.data.resource``

        ::

            加载的资源，包含和资源相关的属性和判断。

    2. ``wordmarker.data.formatter``

        ::

            格式化数据，对某些数据进行处理。

"""
from wordmarker.data.resource import Resource
from wordmarker.data.formatter import SqlFormatter



import csv
from abc import ABCMeta, abstractmethod
from logging import Logger
from typing import Union, List, Dict, Optional, Sequence
import pandas as pd
from pandas import DataFrame, Series
from pandas._libs import lib
from pandas._typing import StorageOptions, Label, IndexLabel, CompressionOptions
from pandas.core.generic import bool_t
from pandas.io.parsers import _c_parser_defaults, TextFileReader
from wordmarker.contexts import YamlContext, SystemContext
from wordmarker.creatives import FactoryBean
from wordmarker.loaders import DefaultResourceLoader
from wordmarker.utils import log, PathUtils


class CsvOperations(metaclass=ABCMeta):
    """
    ::

        csv文件的相关操作的抽象类
    """

    @abstractmethod
    def csv_to_df(self):
        """
        .. note::

            将csv文件转换为 ``DataFrame``

        :return: - 数据框
        """
        pass

    @abstractmethod
    def df_to_csv(self, *args, **kwargs):
        """
        .. note::

            将 ``DataFrame`` 转换为csv文件

        """
        pass


class CsvHelper(DefaultResourceLoader, SystemContext):
    """
    ::

        通过读取配置文件，获取csv文件的相关信息
    """
    __csv_helper = None

    @log
    def __init__(self):
        super().__init__()
        self.__yaml_context: YamlContext = FactoryBean().get_bean("yaml_context")

    def get_csv_in_path(self):
        """
        .. note::

            通过读取yaml文件的 ``data.csv.in.path`` 属性，获取输入的csv文件或目录的绝对路径

        :return: - csv文件或目录的绝对路径
        """
        prop = "data.csv.input.path"
        value = self.__yaml_context.get_value(prop)
        return PathUtils(self.__yaml_context.path, value).get_relative_path()

    def get_csv_out_path(self):
        """
        .. note::

            通过读取yaml文件的 ``data.csv.output.dir`` 属性，获取输出的csv的目录的绝对路径

        :return: - csv目录的绝对路径
        """
        prop = "data.csv.output.dir"
        value = self.__yaml_context.get_value(prop)
        return PathUtils(self.__yaml_context.path, value).get_relative_path()

    def get_csv(self):
        """
        .. note::

            获取csv文件的绝对路径

        :return: - yaml文件中 ``data.csv.input.path`` 是目录，返回当前目录下的所有csv文件的绝对路径

                 - yaml文件中 ``data.csv.input.path`` 是文件，返回当前文件的绝对路径
        """
        return self.get_resource(self.get_csv_in_path()).get_file()

    def get_csv_file_name(self):
        """
        .. note::

            获取csv文件的文件名

        :return: - 是文件，返回文件的名字

                 - 是目录，返回当前目录下的所有文件的文件名
        """
        return self.get_resource(self.get_csv_in_path()).get_file_name()

    def __new__(cls, *args, **kwargs):
        if cls.__csv_helper is None:
            cls.__csv_helper = object.__new__(cls)
        return cls.__csv_helper


class CsvTemplate(CsvOperations, CsvHelper):
    """
    ::

        操作csv文件的模板
    """

    @log
    def __init__(self):
        super().__init__()

    def csv_to_df(self,
                  sep=lib.no_default,
                  delimiter=None,
                  # Column and Index Locations and Names
                  header="infer",
                  names=None,
                  index_col=0,
                  usecols=None,
                  squeeze=False,
                  prefix=None,
                  mangle_dupe_cols=True,
                  # General Parsing Configuration
                  dtype=None,
                  engine=None,
                  converters=None,
                  true_values=None,
                  false_values=None,
                  skipinitialspace=False,
                  skiprows=None,
                  skipfooter=0,
                  nrows=None,
                  # NA and Missing Data Handling
                  na_values=None,
                  keep_default_na=True,
                  na_filter=True,
                  verbose=False,
                  skip_blank_lines=True,
                  # Datetime Handling
                  parse_dates=False,
                  infer_datetime_format=False,
                  keep_date_col=False,
                  date_parser=None,
                  dayfirst=False,
                  cache_dates=True,
                  # Iteration
                  iterator=False,
                  chunksize=None,
                  # Quoting, Compression, and File Format
                  compression="infer",
                  thousands=None,
                  decimal: str = ".",
                  lineterminator=None,
                  quotechar='"',
                  quoting=csv.QUOTE_MINIMAL,
                  doublequote=True,
                  escapechar=None,
                  comment=None,
                  encoding=None,
                  dialect=None,
                  # Error Handling
                  error_bad_lines=True,
                  warn_bad_lines=True,
                  # Internal
                  delim_whitespace=False,
                  low_memory=_c_parser_defaults["low_memory"],
                  memory_map=False,
                  float_precision=None,
                  storage_options: StorageOptions = None,
                  ) -> Union[Union[TextFileReader, Series, DataFrame, None],
                             Dict[str, Union[TextFileReader, Series, DataFrame, None]]]:
        """
        .. note::

            从yaml配置中读取csv文件，转换为 ``pandas`` 的 ``DataFrame``

        .. tip::

            `csv_to_df的参数详情 <https://pandas.pydata.org/docs/reference/api/pandas.read_csv.html#pandas.read_csv>`_ ，
            请访问 `pandas官网`_ ，了解更多信息

        :return: - 多个csv文件，返回一个 ``DataFrame`` 的字典

                    - key为文件名
                    - value为 ``DataFrame``
                 - 一个csv文件，返回一个 ``DataFrame``
        """
        csv_file = self.get_csv()
        csv_dict = {}
        if type(csv_file) is list:
            index = 0
            for i in csv_file:
                csv_dict[self.get_csv_file_name()[index]] = pd.read_csv(i,
                                                                        sep=sep,
                                                                        delimiter=delimiter,
                                                                        header=header,
                                                                        names=names,
                                                                        index_col=index_col,
                                                                        usecols=usecols,
                                                                        squeeze=squeeze,
                                                                        prefix=prefix,
                                                                        mangle_dupe_cols=mangle_dupe_cols,
                                                                        dtype=dtype,
                                                                        engine=engine,
                                                                        converters=converters,
                                                                        true_values=true_values,
                                                                        false_values=false_values,
                                                                        skipinitialspace=skipinitialspace,
                                                                        skiprows=skiprows,
                                                                        skipfooter=skipfooter,
                                                                        nrows=nrows,
                                                                        na_values=na_values,
                                                                        keep_default_na=keep_default_na,
                                                                        na_filter=na_filter,
                                                                        verbose=verbose,
                                                                        skip_blank_lines=skip_blank_lines,
                                                                        parse_dates=parse_dates,
                                                                        infer_datetime_format=infer_datetime_format,
                                                                        keep_date_col=keep_date_col,
                                                                        date_parser=date_parser,
                                                                        dayfirst=dayfirst,
                                                                        cache_dates=cache_dates,
                                                                        iterator=iterator,
                                                                        chunksize=chunksize,
                                                                        compression=compression,
                                                                        thousands=thousands,
                                                                        decimal=decimal,
                                                                        lineterminator=lineterminator,
                                                                        quotechar=quotechar,
                                                                        quoting=quoting,
                                                                        doublequote=doublequote,
                                                                        escapechar=escapechar,
                                                                        comment=comment,
                                                                        encoding=encoding,
                                                                        dialect=dialect,
                                                                        error_bad_lines=error_bad_lines,
                                                                        warn_bad_lines=warn_bad_lines,
                                                                        delim_whitespace=delim_whitespace,
                                                                        low_memory=low_memory,
                                                                        memory_map=memory_map,
                                                                        float_precision=float_precision,
                                                                        storage_options=storage_options)
                index += 1
            return csv_dict
        else:
            return pd.read_csv(csv_file,
                               sep=sep,
                               delimiter=delimiter,
                               header=header,
                               names=names,
                               index_col=index_col,
                               usecols=usecols,
                               squeeze=squeeze,
                               prefix=prefix,
                               mangle_dupe_cols=mangle_dupe_cols,
                               dtype=dtype,
                               engine=engine,
                               converters=converters,
                               true_values=true_values,
                               false_values=false_values,
                               skipinitialspace=skipinitialspace,
                               skiprows=skiprows,
                               skipfooter=skipfooter,
                               nrows=nrows,
                               na_values=na_values,
                               keep_default_na=keep_default_na,
                               na_filter=na_filter,
                               verbose=verbose,
                               skip_blank_lines=skip_blank_lines,
                               parse_dates=parse_dates,
                               infer_datetime_format=infer_datetime_format,
                               keep_date_col=keep_date_col,
                               date_parser=date_parser,
                               dayfirst=dayfirst,
                               cache_dates=cache_dates,
                               iterator=iterator,
                               chunksize=chunksize,
                               compression=compression,
                               thousands=thousands,
                               decimal=decimal,
                               lineterminator=lineterminator,
                               quotechar=quotechar,
                               quoting=quoting,
                               doublequote=doublequote,
                               escapechar=escapechar,
                               comment=comment,
                               encoding=encoding,
                               dialect=dialect,
                               error_bad_lines=error_bad_lines,
                               warn_bad_lines=warn_bad_lines,
                               delim_whitespace=delim_whitespace,
                               low_memory=low_memory,
                               memory_map=memory_map,
                               float_precision=float_precision,
                               storage_options=storage_options
                               )

    def df_to_csv(self,
                  data_dict: Dict[str, DataFrame],
                  sep: str = ",",
                  na_rep: str = "",
                  float_format: Optional[str] = None,
                  columns: Optional[Sequence[Label]] = None,
                  header: Union[bool_t, List[str]] = True,
                  index: bool_t = True,
                  index_label: Optional[IndexLabel] = None,
                  mode: str = "w",
                  encoding: Optional[str] = None,
                  compression: CompressionOptions = "infer",
                  quoting: Optional[int] = None,
                  quotechar: str = '"',
                  line_terminator: Optional[str] = None,
                  chunksize: Optional[int] = None,
                  date_format: Optional[str] = None,
                  doublequote: bool_t = True,
                  escapechar: Optional[str] = None,
                  decimal: str = ".",
                  errors: str = "strict",
                  storage_options: StorageOptions = None,
                  ):
        """
        .. note::

            将一个或多个 ``DataFrame`` 转换成csv文件，输出到指定文件夹

        .. tip::

            `df_to_csv的参数详情 <https://pandas.pydata.org/docs/reference/api/pandas.DataFrame.to_csv.html?highlight=to_csv#pandas.DataFrame.to_csv>`_ ，
            请访问 `pandas官网`_ ，了解更多信息

        :param data_dict: 由 ``DataFrame`` 组成的字典，key为输出的文件名，value为 ``DataFrame``
        """
        d = self.get_csv_out_path()

        if type(data_dict) is dict:
            file_name_list = []
            for key in data_dict.keys():
                file_name_list.append(key)

            data_list = []

            for value in data_dict.values():
                data_list.append(value)

            idx = 0
            for value in data_list:
                value.to_csv(d + self.path_separator + file_name_list[idx],
                             sep=sep,
                             na_rep=na_rep,
                             float_format=float_format,
                             columns=columns,
                             header=header,
                             index=index,
                             index_label=index_label,
                             mode=mode,
                             encoding=encoding,
                             compression=compression,
                             quoting=quoting,
                             quotechar=quotechar,
                             line_terminator=line_terminator,
                             chunksize=chunksize,
                             date_format=date_format,
                             doublequote=doublequote,
                             escapechar=escapechar,
                             decimal=decimal,
                             errors=errors,
                             storage_options=storage_options
                             )
                idx += 1
            self._logger: Logger
            self._logger.info("csv文件输出成功")
        else:
            self._logger.error("输入参数不正确")



import os
from wordmarker.contexts import Context


class SystemContext(Context):
    """
    ::

        系统上下文，获取和系统有关的属性
    """

    def _init(self):
        """
        ::

            初始化系统上下文
        """
        pass

    path_separator = os.sep  # 路径分隔符
    file_separator = os.pathsep  # 文件分隔符



import logging
import coloredlogs


class LoggerFactory(object):
    """
    ::

        logger工厂，获取logger对象
    """

    @staticmethod
    def get_logger(logger_name=None):
        """
        .. note::

            获取日志对象 ``logger``

        :param logger_name: ``logger`` 名字
        :return: - 日志对象 ``logger``
        """
        logger = logging.getLogger(logger_name)
        coloredlogs.install(logging.INFO)
        return logger


def log(fun):
    """
    .. note::

        装饰器，为类注入logger对象

    .. tip::

        在类的 ``__init__`` 方法上加上 ``@log`` ，可以通过类的 ``self._logger`` 获取到 ``logger`` 对象，来打印日志

        你也可以直接使用 ``LoggerFactory`` 获取 ``logger`` 对象，来打印日志

    .. error::

        只能放在类的 ``__init__`` 方法上，不能放在类上，放在类上会修改类的元类

        如果当前类是被继承或继承了某个类，会导致元类冲突的异常，所以目前只支持放在 ``__init__`` 方法上
    """

    def wrapper(self, *args):
        clazz_name = str(self.__class__)
        clazz_name = clazz_name.split("'")[1]
        logger: logging.Logger = LoggerFactory.get_logger(clazz_name)
        self._logger = logger
        return fun(self, *args)

    return wrapper



from abc import ABCMeta, abstractmethod


class AbstractBuilder(metaclass=ABCMeta):
    """
    ::

        用于构造复杂的对象
    """

    @abstractmethod
    def append(self, *args, **kwargs):
        """
        .. note::

            添加对象的部件

        :return: - 当前builder对象
        """
        pass

    @abstractmethod
    def build(self):
        """
        .. note::

            构建复杂对象的实例

        :return: - 复杂对象的实例
        """
        pass



from setuptools import setup, find_packages

with open('README.md', encoding='utf-8') as f:
    long_desc = f.read()

setup(
    name='wordmarker',
    version='0.0.6',
    author='chensixiang',
    author_email='chensixiang1234@gmail.com',
    license_file='LICENSE',
    license='MIT',
    url='https://wordmarker.readthedocs.io',
    download_url='https://pypi.org/project/wordmarker/',
    project_urls={
        "Code": "https://github.com/lostblackknight/wordmarker",
        "Issue tracker": "https://github.com/lostblackknight/wordmarker/issues",
    },
    description='Word documentation generator',
    long_description=long_desc,
    long_description_content_type='text/markdown',
    packages=find_packages(
        exclude=['tests', 'tests.contexts', 'tests.templates', 'tests.utils'],
    ),
    zip_safe=False,
    install_requires=[
        'chardet==4.0.0',
        'coloredlogs==15.0',
        'greenlet==1.0.0',
        'humanfriendly==9.1',
        'numpy==1.19.2',
        'pandas>=1.2.3',
        'psycopg2==2.8.6',
        'pyreadline==2.1',
        'python-dateutil==2.8.1',
        'pytz==2021.1',
        'PyYAML==5.4.1',
        'six==1.15.0',
        'SQLAlchemy==1.4.5',
        'click==7.1.2',
        'ruamel.yaml==0.16.12',
        'ruamel.yaml.clib==0.2.2',
        'docutils>=0.13.1',
        'Pygments>=2.5.1',
        'bleach==3.3.0',
        'docxtpl==0.11.4',
        'Jinja2==2.11.3',
        'lxml==4.6.3',
        'MarkupSafe==1.1.1',
        'python-docx==0.8.10',
    ],
    entry_points={
        'console_scripts': [
            'wordmarker-quickstart = wordmarker.scripts.quickstart:main',
        ],
    },
    python_requires=">=3.7",
)



import yaml
from wordmarker.data import Resource
from wordmarker.loaders import DefaultResourceLoader
from wordmarker.utils import log


class YamlResourceLoader(DefaultResourceLoader):
    """
    ::

        yaml文件的资源加载器
    """

    @log
    def __init__(self):
        super().__init__()

    def load(self, resource: Resource):
        """
        .. note::

            加载yaml资源

        :param resource: yaml资源实例
        :return: - yaml资源为文件，获取文件中的数据

                 - yaml文件资源为目录，获取目录下所有yaml文件中的数据，返回一个字典

                    - key为文件的绝对路径

                    - value为文件的数据
        """
        if resource.exists():
            if resource.is_file():
                try:
                    with open(resource.get_file(), encoding=resource.get_file_encoding()) as f:
                        data = yaml.safe_load(f.read())
                        return data
                except (IOError, ValueError, TypeError) as exc:
                    self._logger.exception("加载文件时发生错误", exc_info=exc)
            else:
                file_list = resource.get_file()
                encoding_dict = resource.get_file_encoding()
                data_dict = {}
                try:
                    for file in file_list:
                        with open(file, encoding=encoding_dict.get(file)) as f:
                            data = yaml.safe_load(f.read())
                            data_dict[file] = data
                    return data_dict
                except (IOError, ValueError, TypeError) as exc:
                    self._logger.exception("加载文件时发生错误", exc_info=exc)
        else:
            self._logger.error("文件或目录不存在")



from abc import ABCMeta, abstractmethod


class ResourceLoader(metaclass=ABCMeta):
    """
    ::

        资源加载器的抽象类
    """

    @abstractmethod
    def get_resource(self, path):
        """
        .. note::

            获取资源

        :param path: 路径
        :return: - 资源
        """
        pass

    @abstractmethod
    def load(self, resource):
        """
        .. note::

            加载资源

        :param resource: 资源
        :return: - 资源中的数据
        """
        pass
